// Programming 2D Games
// Copyright (c) 2013 by: 
// Charles Kelly
// TextureFilterDemo constants.h v1.0
#ifndef _CONSTANTS_H            // Prevent multiple definitions if this 
#define _CONSTANTS_H            // file is included in more than one place
#define WIN32_LEAN_AND_MEAN

#include <windows.h>

//-----------------------------------------------
// Useful macros
//-----------------------------------------------
// Safely delete pointer referenced item
#define SAFE_DELETE(ptr)       { if (ptr) { delete (ptr); (ptr)=NULL; } }
// Safely release pointer referenced item
#define SAFE_RELEASE(ptr)      { if(ptr) { (ptr)->Release(); (ptr)=NULL; } }
// Safely delete pointer referenced array
#define SAFE_DELETE_ARRAY(ptr) { if(ptr) { delete [](ptr); (ptr)=NULL; } }
// Safely call onLostDevice
#define SAFE_ON_LOST_DEVICE(ptr)    { if(ptr) { ptr->onLostDevice(); } }
// Safely call onResetDevice
#define SAFE_ON_RESET_DEVICE(ptr)   { if(ptr) { ptr->onResetDevice(); } }
#define TRANSCOLOR  SETCOLOR_ARGB(0,255,0,255)  // transparent color (magenta)

//-----------------------------------------------
//                  Constants
//-----------------------------------------------
// graphic images
const char TEXTURE1_IMAGE[]   = "pictures\\texture1to1.png";
const char TEXTURE2_IMAGE[]   = "pictures\\texture2to1.png";

// window
const char CLASS_NAME[] = "TextureFilterDemo";
const char GAME_TITLE[] = "Texture Filter Demo";
const bool FULLSCREEN = false;              // ***** DO NOT USE FULLSCREEN IN THIS DEMO *****
const UINT GAME_WIDTH =  160;               // width of game in pixels
const UINT GAME_HEIGHT = 100;               // height of game in pixels
const int  IMAGE_FRAME = 0;                 // frame of texture
const int  TEXTURE_COLS = 2;                // texture has 2 columns
const int  IMAGE_WIDTH = 32;                // width of image
const int  IMAGE_HEIGHT = 32;               // height of image

// game
const double PI = 3.14159265;
const float FRAME_RATE  = 200.0f;               // the target frame rate (frames/sec)
const float MIN_FRAME_RATE = 10.0f;             // the minimum frame rate
const float MIN_FRAME_TIME = 1.0f/FRAME_RATE;   // minimum desired time for 1 frame
const float MAX_FRAME_TIME = 1.0f/MIN_FRAME_RATE; // maximum time used in calculations

// audio files required by audio.cpp
// WAVE_BANK must be location of .xwb file.
const char WAVE_BANK[]  = "";
// SOUND_BANK must be location of .xsb file.
const char SOUND_BANK[] = "";

// audio cues

// key mappings
// In this game simple constants are used for key mappings. If variables were used
// it would be possible to save and restore key mappings from a data file.
const UCHAR CONSOLE_KEY  = VK_OEM_3;    // `~ key for U.S.
const UCHAR ESC_KEY     = VK_ESCAPE;    // escape key
const UCHAR ALT_KEY     = VK_MENU;      // Alt key
const UCHAR ENTER_KEY   = VK_RETURN;    // Enter key

#endif
